// Image.cpp
//

#include "stdafx.h"
#include "Image.h"

#define RECTWIDTH( lpRect )  ( ( lpRect )->right - ( lpRect )->left )
#define RECTHEIGHT( lpRect ) ( ( lpRect )->bottom - ( lpRect )->top )

int DIBNumColors( LPBITMAPINFOHEADER lpbi ) {
    if ( lpbi->biClrUsed != 0 ) {
        return lpbi->biClrUsed;
    }

    switch ( lpbi->biBitCount ) {
    case 1:
        return 2;
    case 4:
        return 16;
    case 8:
        return 256;
    default:
        return 0;
    }
}

CImage::CImage() {
    m_Size = CSize( 0, 0 );
    m_BitCount = 1;
    m_hImage = NULL;
    m_pPal = NULL;
}

void CImage::Free() {
    if ( m_hImage ) {
        GlobalFree( m_hImage );

        m_Size = CSize( 0, 0 );
        m_BitCount = 1;
        m_hImage = NULL;
    }

    if ( m_pPal != NULL ) {
        delete m_pPal;
        m_pPal = NULL;
    }
}

BOOL CImage::CreateGrayImage( BYTE* Data, int Width, int Height ) {
    Free();

    int NewWidth = ( Width + 3 ) / 4 * 4;
    m_DibSize = sizeof( BITMAPINFOHEADER ) + sizeof( RGBQUAD ) * 256 + NewWidth * Height;
    m_BitCount = 8;
    m_Size = CSize( Width, Height );

    if ( ( m_hImage = ( HGLOBAL ) GlobalAlloc( GMEM_MOVEABLE | GMEM_ZEROINIT, m_DibSize ) ) == NULL ) {
        return FALSE;
    }

    LPSTR pDib = ( LPSTR ) GlobalLock( m_hImage );
    LPBITMAPINFO pbmi = ( LPBITMAPINFO ) pDib;

    pbmi->bmiHeader.biSize = sizeof( BITMAPINFOHEADER );
    pbmi->bmiHeader.biWidth = NewWidth;
    pbmi->bmiHeader.biHeight = Height;
    pbmi->bmiHeader.biPlanes = 1;
    pbmi->bmiHeader.biBitCount = 8;
    pbmi->bmiHeader.biCompression = BI_RGB;
    pbmi->bmiHeader.biSizeImage = Width * Height;
    pbmi->bmiHeader.biXPelsPerMeter = 0;
    pbmi->bmiHeader.biYPelsPerMeter = 0;
    pbmi->bmiHeader.biClrUsed = 0;
    pbmi->bmiHeader.biClrImportant = 0;
    for ( int k = 0 ; k < 256 ; k++ ) {
        pbmi->bmiColors[ k ].rgbRed = pbmi->bmiColors[ k ].rgbGreen = pbmi->bmiColors[ k ].rgbBlue = BYTE( k );
    }

    BYTE* Buffer = ( BYTE* ) ( pDib + sizeof( BITMAPINFOHEADER ) + sizeof( RGBQUAD ) * 256 );
    BYTE* TempData = Data + Width * ( Height - 1 );
    for ( int i = Height - 1 ; i >= 0 ; i-- ) {
        memcpy( Buffer, TempData, Width );
        Buffer += NewWidth;
        TempData -= Width;
    }

    GlobalUnlock( m_hImage );

    return TRUE;
}

void CImage::Draw( HDC hDC, LPRECT lpDIBRect, LPRECT lpDCRect ) {
    HPALETTE hOldPal = NULL;

    LPSTR lpDIBHdr  = ( LPSTR ) GlobalLock( m_hImage );
    LPBITMAPINFOHEADER lpbmi = ( LPBITMAPINFOHEADER ) lpDIBHdr;
    LPSTR lpDIBBits = lpDIBHdr + sizeof( BITMAPINFOHEADER ) + DIBNumColors( lpbmi ) * sizeof( RGBQUAD );

    if ( m_pPal != NULL ) {
        hOldPal = SelectPalette( hDC, ( HPALETTE ) m_pPal->m_hObject, TRUE );
    }

    if ( ( RECTWIDTH( lpDCRect ) == RECTWIDTH( lpDIBRect ) ) 
        && ( RECTHEIGHT( lpDCRect ) == RECTHEIGHT( lpDIBRect ) ) ) {
        SetDIBitsToDevice( hDC,
            lpDCRect->left,
            lpDCRect->top,
            RECTWIDTH( lpDCRect ),
            RECTHEIGHT( lpDCRect ),
            lpDIBRect->left,
//            lpDIBRect->top,
            lpbmi->biHeight - lpDIBRect->top - RECTHEIGHT( lpDIBRect ),
            0,
            lpbmi->biHeight,
            lpDIBBits,
            ( LPBITMAPINFO ) lpDIBHdr,
            DIB_RGB_COLORS );
    }
    else {
        SetStretchBltMode( hDC, HALFTONE );

        StretchDIBits( hDC,
            lpDCRect->left,
            lpDCRect->top,
            RECTWIDTH( lpDCRect ),
            RECTHEIGHT( lpDCRect ),
            lpDIBRect->left,
//            lpDIBRect->top,
            lpbmi->biHeight - lpDIBRect->top - RECTHEIGHT( lpDIBRect ),
            RECTWIDTH( lpDIBRect ),
            RECTHEIGHT( lpDIBRect ),
            lpDIBBits,
            ( LPBITMAPINFO ) lpDIBHdr,
            DIB_RGB_COLORS,
            SRCCOPY );
    }

    GlobalUnlock( m_hImage );

    if ( hOldPal != NULL ) {
        SelectPalette( hDC, hOldPal, TRUE );
    }
}

double CImage::Draw( CDC* pDC, CRect& Rect, BOOL bStretch ) {
    int cx = Rect.Width();
    int cy = Rect.Height();

    int ImgW = GetWidth();
    int ImgH = GetHeight();

    double scale = min( ( float ) cx / ImgW, ( float ) cy / ImgH );

    if ( !bStretch ) {
        scale = min( 1.0, scale );
    }

    Rect.DeflateRect( ( cx - ( int ) ( ImgW * scale ) ) / 2, ( cy - ( int ) ( ImgH * scale ) ) / 2 );

    Draw( pDC->GetSafeHdc(), GetRect(), Rect );

    return scale;
}

