#include <stdlib.h>
#include <stdio.h>

#define BITMAPFILESIGNATURE 0x4D42

typedef enum tagIMAGETYPE
	{IMAGETYPEMONOCHROME,IMAGETYPEGRAYSCALE,IMAGETYPEINDEX,IMAGETYPERGB,IMAGETYPECMYK} IMAGETYPE;

typedef struct tagBITMAPFILEHEADER {
	unsigned short	bfType;
	unsigned long	bfSize;
	unsigned short	bfReserved1;
	unsigned short	bfReserved2;
	unsigned long	bfOffBits;
} BITMAPFILEHEADER;

#define BI_RGB			0L
#define BI_RLE8			1L
#define BI_RLE4			2L
#define BI_BITFIELDS	3L

typedef struct tagBITMAPINFOHEADER {
	unsigned long	biSize;
	long	biWidth;
	long	biHeight;
	unsigned short	biPlanes;
	unsigned short	biBitCount;
	unsigned long	biCompression;
	unsigned long	biSizeImage;
	long	biXPelsPerMeter;
	long	biYPelsPerMeter;
	unsigned long	biClrUsed;
	unsigned long	biClrImportant;
} BITMAPINFOHEADER;

typedef struct tagRGBQUAD {
	unsigned char	rgbBlue;
	unsigned char	rgbGreen;
	unsigned char	rgbRed;
	unsigned char	rgbReserved;
} RGBQUAD;

typedef struct tagRGBTRIPLE {
	unsigned char	rgbtBlue;
	unsigned char	rgbtGreen;
	unsigned char	rgbtRed;
} RGBTRIPLE;

unsigned char RGBQuadToGrayLevel(RGBQUAD* pRGBQuad)
{
	return (pRGBQuad->rgbBlue+pRGBQuad->rgbGreen+pRGBQuad->rgbRed)/3; 
}

unsigned char RGBTripleToGrayLevel(RGBTRIPLE* pRGBTriple)
{
	return (pRGBTriple->rgbtBlue+pRGBTriple->rgbtGreen+pRGBTriple->rgbtRed)/3; 
}

int LoadBMPImage( const char* Name, int *pWidth, int *pHeight, unsigned char **pImage ) {
	FILE *f;
	BITMAPFILEHEADER BFH;
	BITMAPINFOHEADER BIH;
	IMAGETYPE ImageType;
	int ColorsInPalette;
	RGBQUAD *Palette;
	unsigned long Size,WidthBytes,LSize;
	int I,J;
	unsigned char *Bits;
	unsigned char *S,*SP;
	int SBPP,SBPPAdd;
	unsigned char IndexMask;
    unsigned char* TempImage;

	f=fopen(Name,"rb");

	fread(&BFH,sizeof(BFH),1,f);
	if(BFH.bfType!=BITMAPFILESIGNATURE) return 0;
	fseek(f,14,SEEK_SET);  //Because BFH structure must be 2 bytes packed (14 bytes, not 16)
	fread(&BIH,sizeof(BIH),1,f);
	if(BIH.biSize!=sizeof(BIH)) return 0;
	switch(BIH.biBitCount)
	{
		case 1:
			ColorsInPalette=2;
			ImageType=IMAGETYPEINDEX;
			break;
		case 4:
			ColorsInPalette=16;
			ImageType=IMAGETYPEINDEX;
			break;
		case 8:
			ColorsInPalette=256;
			ImageType=IMAGETYPEINDEX;
			break;
		case 24:
        case 32:
			ColorsInPalette=0;
			ImageType=IMAGETYPERGB;
			break;
		default:
			return 0;
	}
	if(ColorsInPalette>=0 && BIH.biClrUsed>0) ColorsInPalette=BIH.biClrUsed;
	if(BIH.biCompression!=BI_RGB) return 0;
	Size=(unsigned long)(abs(BIH.biHeight))*((BIH.biBitCount*(unsigned long)(BIH.biWidth))/32
		+(unsigned long)(((BIH.biBitCount*(unsigned long)(BIH.biWidth))%32>0)?1:0))*4;
	if(BIH.biWidth==0 || BIH.biHeight==0) return 0;
	WidthBytes=BIH.biBitCount*BIH.biWidth;
	WidthBytes=(WidthBytes/32+(unsigned long)((WidthBytes%32>0)?1:0))*4;
	LSize=WidthBytes*(unsigned long)(abs(BIH.biHeight));
	Bits=(unsigned char*)malloc(LSize);
	Palette=(RGBQUAD*)malloc(ColorsInPalette*sizeof(RGBQUAD));
	fread(Palette,sizeof(RGBQUAD),ColorsInPalette,f);
	fread(Bits,Size,1,f);
	fclose(f);

    *pImage = malloc( BIH.biWidth * abs( BIH.biHeight ) );

    if(BIH.biHeight<0) S=Bits;
	else S=Bits+(BIH.biHeight-1)*WidthBytes;
	SBPP=BIH.biBitCount;
	SBPPAdd=8-SBPP;
	IndexMask=0xFF>>SBPPAdd;

    TempImage = *pImage;

	for(I=0; I<BIH.biHeight; I++)
	{
		for(J=0; J<BIH.biWidth; J++)
		{
			SP=S+J*BIH.biBitCount/8;
			switch(ImageType)
			{
				case IMAGETYPEMONOCHROME:
					TempImage[J]=((*SP)>>(7-J%8)&0x01)*255;
					break;
				case IMAGETYPEGRAYSCALE:
					TempImage[J]=(*SP);
					break;
				case IMAGETYPEINDEX:
					TempImage[J]=RGBQuadToGrayLevel(Palette+((*SP)>>(SBPPAdd-J*SBPP%8)&IndexMask));
					break;
				case IMAGETYPERGB:
					TempImage[J]=RGBTripleToGrayLevel((RGBTRIPLE*)(SP));
					break;
			}
		}
		if(BIH.biHeight<0) S+=WidthBytes;
		else S-=WidthBytes;

        TempImage += BIH.biWidth;
	}
	*pWidth=BIH.biWidth;
	*pHeight=abs(BIH.biHeight);
	free((void*)Bits);
	free((void*)Palette);

    return 1;
}
